@@ -28,7 +28,7 @@ from object_detection.protos import pipeline_pb2
 from object_detection import model_lib_v2
 from tensorflow.python.summary.summary_iterator import summary_iterator
 from google.protobuf import text_format
-
+from object_detection.protos import optimizer_pb2
 
 def create_record_files(dict_annotations, train_list, train_list_id, eval_list, 
                         eval_list_id, label_path, record_dir, tfExample_generator, annotation_type):
@@ -137,14 +137,14 @@ def check_batch_size(config_dict):
         print("This mean that all your images should have the same shape. If not then set batch size to 1 or change the image resizer to a fixed_shape_resizer.")
         
     #image_resizer.HasField("fixed_shape_resizer"):
-    
+
 def configure_learning_rate(configs, learning_rate=None, parameters={}):
     keys = parameters.keys()
 
     if 'lr_type' in keys:
         lr_type = parameters['lr_type']
     else:
-        lr_type = 'constant_learning_rate'
+        lr_type = 'constant'
     if 'decay_steps' in keys:
         decay_steps = parameters['decay_steps']
     else:
@@ -169,8 +169,23 @@ def configure_learning_rate(configs, learning_rate=None, parameters={}):
         total_steps = parameters['total_steps']
     else:
         total_steps = None
+    if 'momentum' in keys:
+        momentum = parameters['momentum']
+    else:
+        momentum = 0.9
+    if 'optimizer' in keys:
+        optimizer = parameters['optimizer']
+        if optimizer == "rms_prop":
+            optimizer_type = "rms_prop_optimizer"
+        elif optimizer == "momentum":
+            optimizer_type = "momentum_optimizer"
+        elif optimizer == "adam":
+            optimizer_type = "adam_optimizer"
+        else:
+            raise TypeError("Optimizer %s is not supported." % optimizer)
+    else:
+        optimizer_type = configs["train_config"].optimizer.WhichOneof("optimizer")
 
-    optimizer_type = configs["train_config"].optimizer.WhichOneof("optimizer")
     if optimizer_type == "rms_prop_optimizer":
         optimizer_config = configs["train_config"].optimizer.rms_prop_optimizer
     elif optimizer_type == "momentum_optimizer":
@@ -179,33 +194,46 @@ def configure_learning_rate(configs, learning_rate=None, parameters={}):
         optimizer_config = configs["train_config"].optimizer.adam_optimizer
     else:
         raise TypeError("Optimizer %s is not supported." % optimizer_type)
+    
+    if optimizer_type == "rms_prop_optimizer" or optimizer_type == "momentum_optimizer":
+        optimizer_config.momentum_optimizer_value = min(max(0.0, momentum), 1.0)
 
-    print(optimizer_config)
     if lr_type == "constant":
         optimizer_config.learning_rate.constant_learning_rate.learning_rate = learning_rate
-        print(optimizer_config)
     elif lr_type == "exponential_decay":
         if decay_steps != None and decay_factor != None:
             optimizer_config.learning_rate.exponential_decay_learning_rate.initial_learning_rate = learning_rate
             optimizer_config.learning_rate.exponential_decay_learning_rate.decay_steps = decay_steps
             optimizer_config.learning_rate.exponential_decay_learning_rate.decay_factor = decay_factor
             optimizer_config.learning_rate.exponential_decay_learning_rate.staircase = staircase
-            print(optimizer_config)
     elif lr_type == "manual_step":
-        optimizer_config.learning_rate["manual_step_learning_rate"] = {}
-        manual_lr = optimizer_config.learning_rate.manual_step_learning_rate
-        original_learning_rate = manual_lr.initial_learning_rate
-        learning_rate_scaling = float(learning_rate) / original_learning_rate
-        manual_lr.initial_learning_rate = learning_rate
-        for schedule in manual_lr.schedule:
-            schedule.learning_rate *= learning_rate_scaling
+        optimizer_config.learning_rate.manual_step_learning_rate.initial_learning_rate = learning_rate
+        steps = []
+        lrs = []
+        for k in parameters.keys():
+            if k[:-1] == 'step_':
+                step = int(k[-1])
+                if 'lr_' + str(step) in parameters.keys():
+                    steps.append(parameters[k])
+                    lrs.append(parameters['lr_' + str(step)])
+        zipped = zip(steps, lrs)
+        schedules = sorted(zipped, key = lambda x: x[0]) 
+        for i, schedule in enumerate(schedules):
+            print(schedule)
+            sched = optimizer_config.learning_rate.manual_step_learning_rate.schedule
+            print(sched)
+            # sched.step = schedule[0]
+            sched.learning_rate = schedule[1]
+            optimizer_config.learning_rate.manual_step_learning_rate.schedule.append(sched)
+            # optimizer_config.learning_rate.manual_step_learning_rate.schedule.step = schedule[0]
+            # optimizer_config.learning_rate.manual_step_learning_rate.schedule.learning_rate = schedule[1]
         print(optimizer_config)
     elif lr_type == "cosine_decay":
         optimizer_config.learning_rate.cosine_decay_learning_rate.learning_rate_base = learning_rate
         optimizer_config.learning_rate.cosine_decay_learning_rate.total_steps = total_steps
         optimizer_config.learning_rate.cosine_decay_learning_rate.warmup_learning_rate = warmup_lr
         optimizer_config.learning_rate.cosine_decay_learning_rate.warmup_steps = warmup_steps
-        print(optimizer_config)
+    print(optimizer_config)
 
 def set_image_resizer(config_dict, shape):
     '''
@@ -373,7 +401,6 @@ def edit_config(model_selected, input_config_dir, output_config_dir, num_steps,
                             eval_record_path=os.path.join(record_dir,"eval.record"))
 
     if learning_rate is not None:
-        # config_util._update_initial_learning_rate(configs, learning_rate)
         configure_learning_rate(configs, learning_rate, parameters)
     if batch_size is not None:
         config_util._update_batch_size(configs, batch_size)
